package mailbox;

import agent.Agent;

import java.util.Vector;


/**
 * Implementierung der Agenten-Mailbox mit einer FIFO-Queue
 * User: Standard
 * Date: 26.10.2003
 * Time: 15:31:36
 *
 */
public class AgentMailbox {

    private Agent ivAgent;
    private MailboxSystem ivMailboxSystem;
    private Vector ivMessageQueue;

    public AgentMailbox(Agent agent, MailboxSystem mailboxSystem){
        ivAgent = agent;
        ivMailboxSystem = mailboxSystem;
        ivMessageQueue = new Vector();
    }

    /**
     * Sendet die bergebene Nachricht an den Server. Dabei wird sie mit
     * dem Namen des Absenders versehen.
     * @param msg
     */
    public void sendMessageToServer(Message msg){
        msg.setAbsender(ivAgent.getName());
        ivMailboxSystem.sendMessageToServer(msg);
    }

    /**
     * gibt die nchste Nachricht in der Message-Queuezurck oder null, falls
     * keine vorhanden ist.
     * @return
     */
    public synchronized Message getNextMessage(){
        Message lvMessage = null;
        if (ivAgent.isRunning()){

            if(ivMessageQueue.size() == 0){
                 // tue explizit nichts, lvMessage  bleibt auf null
            } else {
                lvMessage = (Message)  ivMessageQueue.remove(0);
            }
        }
        return lvMessage;
    }

    /**
     * Hngt eine Nachricht an die Message-Queue an
     * @param msg
     */
    public void appendMessage(Message msg){
        ivMessageQueue.add(msg);
    }

    /**
     * Setter fr den Namen des Besitzers der Mailbox
     * @return Name des Besitzers der Mailbox
     */
    public String getOwnerName(){
        return ivAgent.getName();
    }

    /**
     * Gibt eine Referenz auf den Agenten zurck, dem die Mailbox gehrt
     * @return
     */
    public Agent getOwner(){
        return ivAgent;
    }


    /**
     * Sendet die bergebene Nachricht alle Agenten. Dabei wird sie mit
     * dem Namen des Absenders versehen.
     * @param msg
     */
     public synchronized void sendBroadcastToAgents(Message msg){
         msg.setAbsender(getOwnerName());
         ivMailboxSystem.sendBroadcastBetweenAgents(msg);
     }

    /**
     * Sendet die bergebene Nachricht den bezeichneten Agenten. Dabei wird sie mit
     * dem Namen des Absenders versehen.
     * @param msg
     */
    public synchronized void sendMessageToAgent(Message msg, String adresse){
        msg.setAbsender(getOwnerName());
        msg.setEmpfaenger(adresse);
        ivMailboxSystem.sendAgentMessageToAgent(msg);
    }





}
